/** @file modbuscli.c MODBUS/UDP client sample for the MSXE1701
 *
 * FC16 queries are for setting things, FC3 for fetching values
 *
 * This code doesn't handle timeouts.
 */

#ifdef WIN32
#include "../../Interface_Library/msxe170x_modbus_clientlib.h"
#else
#include <msxe170x_modbus_clientlib.h>
#endif

#ifdef WIN32
#include <windows.h>
#define sleep Sleep
#else
#include <unistd.h>
#endif // WIN32

#include <modbusclient.h>
#include <commonclient.h>

#include <addidata_network.h>
#include <addidata_dataclient.h>
#include <addidata_time.h>
#include <addidata_console.h>

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterInit(struct modbus * modbus, uint32_t ulMFModuleIndex)
{

    //variables declaration

    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulCounterMode = 0x1;
    uint32_t ulCounterOption = 0;
    uint32_t ulOption01 = 0;
    uint32_t ulOption02 = 0;
    uint32_t ulOption03 = 0;
    uint32_t ulOption04 = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterInit(modbus,ulMFModuleIndex,ulCounterMode,ulCounterOption,ulOption01,ulOption02,ulOption03,ulOption04,&CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterInit" );

}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterRead32BitsValue(struct modbus * modbus, uint32_t ulMFModuleIndex, uint32_t *pulValue, uint32_t *pulTimeStampLow, uint32_t *pulTimeStampHigh)
{
    //variables declaration
    struct MSXE17xx__MFIncCounterRead32BitsValue0_parameters_t Response0;
    struct MSXE17xx__MFIncCounterRead32BitsValue1_parameters_t Response1;
    struct MSXE17xx__MFIncCounterRead32BitsValue2_parameters_t Response2;
    struct MSXE17xx__MFIncCounterRead32BitsValue3_parameters_t Response3;
    int ret = 0;

    switch (ulMFModuleIndex)
    {
        case 0:
            memset(&Response0,0,sizeof(Response0));

            ret = modbus_call_MSXE17xx__MFIncCounterRead32BitsValue0(modbus,&Response0);

            if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFIncCounterRead0"))
            {
                return -1;
            }

            *pulValue         = Response0.ulValue;
            *pulTimeStampLow  = Response0.ulTimeStampLow;
            *pulTimeStampHigh = Response0.ulTimeStampHigh;
            break;

        case 1:
            memset(&Response1,0,sizeof(Response1));

            ret = modbus_call_MSXE17xx__MFIncCounterRead32BitsValue1(modbus,&Response1);

            if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFIncCounterRead1"))
            {
                return -1;
            }

            *pulValue         = Response1.ulValue;
            *pulTimeStampLow  = Response1.ulTimeStampLow;
            *pulTimeStampHigh = Response1.ulTimeStampHigh;
            break;

        case 2:
            memset(&Response2,0,sizeof(Response2));

            ret = modbus_call_MSXE17xx__MFIncCounterRead32BitsValue2(modbus,&Response2);

            if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFIncCounterRead2"))
            {
                return -1;
            }

            *pulValue         = Response2.ulValue;
            *pulTimeStampLow  = Response2.ulTimeStampLow;
            *pulTimeStampHigh = Response2.ulTimeStampHigh;
            break;

        case 3:
            memset(&Response3,0,sizeof(Response3));

            ret = modbus_call_MSXE17xx__MFIncCounterRead32BitsValue3(modbus,&Response3);

            if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE17xx__MFIncCounterRead3"))
            {
                return -1;
            }

            *pulValue         = Response3.ulValue;
            *pulTimeStampLow  = Response3.ulTimeStampLow;
            *pulTimeStampHigh = Response3.ulTimeStampHigh;
            break;
    }


    printf("Counter/position: %d\n",*pulValue);
    printf("Time stamp low  : %u\n", *pulTimeStampLow);
    printf("Time stamp high : %u\n", *pulTimeStampHigh);

    return 0;
}


//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterWrite32BitValue(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulCounterValue = 1000;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterWrite32BitValue(modbus, ulMFModuleIndex, ulCounterValue, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterWrite32BitValue" );
}


//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterClear(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterClear(modbus, ulMFModuleIndex, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterClear" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterRelease(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterRelease(modbus, ulMFModuleIndex, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterRelease" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterInitAndEnableLatch(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulLatchSource      = 2;
    uint32_t ulCondition        = 0;
    uint32_t ulAutoMode         = 1;
    uint32_t ulLatchDigValue    = 0; //Option01 used for LatchDigValue: HO 28.06.12
    uint32_t ulOption02         = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterInitAndEnableLatch(modbus, ulMFModuleIndex, ulLatchSource, ulCondition, ulAutoMode, ulLatchDigValue, ulOption02, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterInitAndEnableLatch" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterDisableAndReleaseLatch(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulLatchSource = 2;
    uint32_t ulOption01    = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterDisableAndReleaseLatch(modbus, ulMFModuleIndex, ulLatchSource, ulOption01, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterDisableAndReleaseLatch" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterInitAndEnableClear(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulClearSource = 2;
    uint32_t ulCondition   = 0;
    uint32_t ulAutoMode    = 1;
    uint32_t ulOption01    = 0;
    uint32_t ulOption02    = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterInitAndEnableClear(modbus, ulMFModuleIndex, ulClearSource, ulCondition, ulAutoMode, ulOption01, ulOption02, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterInitAndEnableClear" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterDisableAndReleaseClear(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulClearSource = 2;
    uint32_t ulOption01    = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterDisableAndReleaseClear(modbus, ulMFModuleIndex, ulClearSource, ulOption01, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterDisableAndReleaseClear" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterInitAndEnableCompareLogic(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulValue          = 50;
    uint32_t ulMode           = 1; // Modulo
    uint32_t ulSynchroTrigger = 0;
    uint32_t ulLatchDigValue  = 0; //Option01 used for LatchDigValue: HO 28.06.12
    uint32_t ulOption02       = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterInitAndEnableCompareLogic(modbus, ulMFModuleIndex, ulValue, ulMode, ulSynchroTrigger, ulLatchDigValue, ulOption02, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterInitAndEnableCompareLogic" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterDisableAndReleaseCompareLogic(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterDisableAndReleaseCompareLogic(modbus, ulMFModuleIndex, &CommandStatus);


    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterDisableAndReleaseCompareLogic" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterInitHardwareTrigger(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulEdgeSelection  = 1; // Rising front
    uint32_t ulCount          = 1;
    uint32_t ulOption01       = 0;
    uint32_t ulOption02       = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterInitHardwareTrigger(modbus, ulMFModuleIndex, ulEdgeSelection, ulCount, ulOption01, ulOption02, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterInitHardwareTrigger" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterReleaseHardwareTrigger(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulOption01 = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterReleaseHardwareTrigger(modbus, ulMFModuleIndex, ulOption01, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterReleaseHardwareTrigger" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterInitIndex(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulEdgeSelection  = 1; // Rising front
    uint32_t ulOption01       = 0;
    uint32_t ulOption02       = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterInitIndex(modbus, ulMFModuleIndex, ulEdgeSelection, ulOption01, ulOption02, &CommandStatus);

    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterInitIndex" );
}

//---------------------------------------------------------------------------------
int sample_MSXE17xx__MFIncCounterReleaseIndex(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    //variables declaration
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = 0;

    uint32_t ulOption01 = 0;

    //Remote function call
    ret = modbus_call_MSXE17xx__MFIncCounterReleaseIndex(modbus, ulMFModuleIndex, ulOption01, &CommandStatus);


    //Return value check
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx__MFIncCounterReleaseIndex" );
}

int sample_MSXE17xx__MFIncCounterInitAndEnableFrequencyMeasurement(struct modbus * modbus, uint32_t ulMFModuleIndex, uint32_t ulTimingInterval, uint32_t ulOption01, uint32_t ulOption02)
{
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = modbus_call_MSXE17xx__MFIncCounterInitAndEnableFrequencyMeasurement(modbus, ulMFModuleIndex, ulTimingInterval, ulOption01, ulOption02, &CommandStatus);
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "sample_MSXE17xx__MFIncCounterInitAndEnableFrequencyMeasurement");
}

int sample_MSXE17xx__MFIncCounterDisableAndReleaseFrequencyMeasurement(struct modbus * modbus, uint32_t ulMFModuleIndex)
{
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = modbus_call_MSXE17xx__MFIncCounterDisableAndReleaseFrequencyMeasurement(modbus, ulMFModuleIndex, 0, &CommandStatus);
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE17xx_MFIncCounterDisableAndReleaseFrequencyMeasurement");
}

int sample_TriggeredFrequencyMeasure_MXCommon__InitAndStartSynchroTimerEx(struct modbus * modbus, uint32_t ulTimeBase, uint32_t ulReloadValue, uint32_t ulNbrOfCycle)
{
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = modbus_call_MXCommon__InitAndStartSynchroTimerEx(modbus, ulTimeBase, ulReloadValue, ulNbrOfCycle, 0, 0, 1, 1, 0, &CommandStatus);
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "sample_MXCommon__InitAndStartSynchroTimerEx");
}

int sample_TriggeredFrequencyMeasure_MXCommon__StopAndReleaseSynchroTimerEx(struct modbus * modbus)
{
    struct GetLastCommandStatus_parameters_t CommandStatus;
    int ret = modbus_call_MXCommon__StopAndReleaseSynchroTimerEx(modbus, 0, &CommandStatus);
    return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "sample_MXCommon__StopAndReleaseSynchroTimerEx");
}

//---------------------------------------------------------------------------------
static void usage(char* name)
{
    printf("%s ADDRESS tcp|udp [Module index: 0 or 1 or 2 or 3] [Sample number] [Sample option 1] [Sample option 2]\n",name);
    printf("Sample number:\n");
    printf("  0: Compare logic\n");
    printf("  1: FrequencyMeasure [Option1 0: One shot / 1: Triggered] [Option2 0: 100 ns / 1: 100 us time base]\n");
    printf("  2: LatchRegister\n");
    printf("  3: CyclicAcquisition\n");
    printf("  4: Clear, Write 1000 and Read\n");
}

static int dataServerFunction(struct modbus * modbus, char *address, uint32_t ulMFModuleIndex, uint32_t dataCount, uint32_t ulTimingParameter, uint32_t ulOption01, uint32_t ulOption02)
{
    int fd;
    void *data = NULL;
    key_thread_handle_t kt;
    unsigned int stop = 0;
    wait_key_input_thread_parameters_t ktparams;
    unsigned long ulLatchCpt = 0;

    /* Connect to data server */
    switch (addidata_dataclient_tcp_connect(address, 0, &fd))
    {
        case 0:
            break;
        case -1:
        addidata_network_perror("socket");
        return -1;
        case -2:
        addidata_network_perror("connect");
        return -2;
        case -3:
        printf("error while parsing address\n");
        return -3;
        case -4:
        addidata_network_perror("WSACleanup() failed");
        return -4;
    }

    data = malloc(100);
    if (data == NULL)
        return -1;

    ktparams.character = 27;
    ktparams.flag = &stop;
    kt = addidata_start_key_thread(&ktparams);

    if (ulTimingParameter != 0)
    {
        int returnValue = 0;

        // If trigger mode is used, start a synchro timer to trigger the frequency measure
        if (ulOption01 == 1)
        {
        	uint32_t ulTimeBase = 1; // ms
        	uint32_t ulReloadValue = 5; // 5 ms
        	uint32_t ulNbrOfCycle = 0; // Continuous

        	returnValue = sample_TriggeredFrequencyMeasure_MXCommon__InitAndStartSynchroTimerEx(modbus, ulTimeBase, ulReloadValue, ulNbrOfCycle);
            if (returnValue != 0)
            {
                printf("\nsample_TriggeredFrequencyMeasure_MXCommon__InitAndStartSynchroTimerEx error %d", returnValue);
                return -3;
            }
        }

        returnValue = sample_MSXE17xx__MFIncCounterInitAndEnableFrequencyMeasurement(modbus, ulMFModuleIndex, ulTimingParameter, ulOption01, ulOption02);
        if (returnValue != 0)
        {
            printf("\nsample_MSXE17xx_MFIncCounterInitAndEnableFrequencyMeasurement error %d", returnValue);
            return -3;
        }
    }

    while (1)
    {
        if (-1 == addidata_dataclient_recvall(fd, data, 5))
        {
            free(data);
            data = NULL;
            return -2;
        }

        fflush(stdout);
        switch (((uint32_t *)data)[3])
        {
            case 0:
                printf("\rCompare occurs on counter %d : %d                                         ", ((uint32_t *)data)[2], ((uint32_t *)data)[4]);
                break;
            case 1:
            {
                double value = ((int *)data)[4];
                if (value < 0)
                    value = -value;

                printf("\rFrequency = %f                                                        ", 1.0 / (((((double)ulTimingParameter * 100) / value) / (ulOption02 == 0 ? 1e9 : 1e6))));
                if (ulOption01 == 0)
                	return 0;
                else
                	break;
            }
            break;
            case 2:
                printf("\rIndex %d occurs on counter low = %d                                       ", ((uint32_t *)data)[2], ((uint32_t *)data)[4]);
                break;

            case 3:
                printf("\rLatch occurs : latch count %lu Counter %d latch value %d                                                                  ", ++ulLatchCpt, ((uint32_t *)data)[2], ((uint32_t *)data)[4]);
                break;
        }

        if (stop)
            break;
    }
    addidata_stop_key_thread(kt);

    if ((ulTimingParameter != 0) && (ulOption01 == 1))
    {
    	int returnValue = sample_TriggeredFrequencyMeasure_MXCommon__StopAndReleaseSynchroTimerEx(modbus);
        if (returnValue != 0)
        {
            printf("\nsample_TriggeredFrequencyMeasure_MXCommon__StopAndReleaseSynchroTimerEx error %d", returnValue);
            return -3;
        }
    }

    free(data);
    data = NULL;
    return 0;
}

//---------------------------------------------------------------------------------
int main(int argc, char** argv)
{
    unsigned int  i               = 0;
    uint32_t      ulMFModuleIndex = 0;
    uint32_t      ulValue         = 0;
    uint32_t      ulTimeStampLow  = 0;
    uint32_t      ulTimeStampHigh = 0;
    int           contype         = 0;
    int           sampleNumber    = 0;
    int           sampleOption01  = 0;
    int           sampleOption02  = 0;
    int           returnValue     = 0;
    struct modbus modbus;

    if (argc < 6)
    {
        usage(argv[0]);
        exit(1);
    }

    if (0 == strcmp(argv[2],"tcp"))
    {
        contype=SOCK_STREAM;
    }
    else if(0 == strcmp(argv[2],"udp"))
    {
        contype=SOCK_DGRAM;
    }
    else
    {
        usage(argv[0]);
        exit(1);
    }

    ulMFModuleIndex = (uint32_t) (argv[3][0] - '0');
    if (ulMFModuleIndex > 3)
    {
        usage(argv[0]);
        exit(1);
    }

    sampleNumber = (uint32_t) (argv[4][0]  - '0');
    if (sampleNumber > 5)
    {
        usage(argv[0]);
        exit(1);
    }

    sampleOption01 = (uint32_t) (argv[5][0]  - '0');
    if (sampleOption01 > 1)
    {
        usage(argv[0]);
        exit(1);
    }

    sampleOption02 = (uint32_t) (argv[6][0]  - '0');
    if (sampleOption02 > 1)
    {
        usage(argv[0]);
        exit(1);
    }

    modbus_init(&modbus);

    switch( modbus_connect(&modbus, argv[1], contype, 0) )
    {
        case 0: /* success */
            break;
        case -1: /* system error */
        addidata_network_perror("modbus_connect");
        exit(1);
        case -2: /* parameter is incorrect */
        printf("modbus_connect: incorrect parameter\n");
        exit(1);
        case -3: /* incorrect address */
        printf("incorrect address\n");
        exit(1);
    }

    returnValue = sample_MSXE17xx__MFIncCounterInit(&modbus, ulMFModuleIndex);
    if (returnValue != 0)
    {
        printf("\nsample_MSXE17xx__MFIncCounterInit Error %d", returnValue);
        goto sample_end;
    }

    returnValue = sample_MSXE17xx__MFIncCounterClear(&modbus, ulMFModuleIndex);
    if (returnValue != 0)
    {
        printf("\nsample_MSXE17xx__MFIncCounterClear");
        goto sample_end;
    }

    returnValue = sample_MSXE17xx__MFIncCounterInitIndex(&modbus, ulMFModuleIndex);
    if (returnValue != 0)
    {
        printf("\nsample_MSXE17xx__MFIncCounterInitIndex");
        goto sample_end;
    }

    returnValue = sample_MSXE17xx__MFIncCounterInitHardwareTrigger(&modbus, ulMFModuleIndex);
    if (returnValue != 0)
    {
        printf("\nsample_MSXE17xx__MFIncCounterInitHardwareTrigger");
        goto sample_end;
    }

    returnValue = sample_MSXE17xx__MFIncCounterWrite32BitValue(&modbus, ulMFModuleIndex);
    if (returnValue != 0)
    {
        printf("\nsample_MSXE17xx__MFIncCounterWrite32BitValue");
        goto sample_end;
    }

    // Compare logic sample
    if (sampleNumber == 0)
    {
        returnValue = sample_MSXE17xx__MFIncCounterInitAndEnableCompareLogic(&modbus, ulMFModuleIndex);
        if (returnValue != 0)
        {
            printf("\nsample_MSXE17xx__MFIncCounterInitAndEnableCompareLogic error %d", returnValue);
            goto sample_end;
        }
        printf("\nsample_MSXE17xx__MFIncCounterInitAndEnableCompareLogic OK");

        returnValue = dataServerFunction(&modbus, argv[1], ulMFModuleIndex, 6 * sizeof(uint32_t), 0, sampleOption01, sampleOption02);
        if (returnValue != 0)
            printf("\ndataServerFunction error %d", returnValue);

        returnValue = sample_MSXE17xx__MFIncCounterDisableAndReleaseCompareLogic(&modbus, ulMFModuleIndex);
        if (returnValue != 0)
        {
            printf("\nsample_MSXE17xx__MFIncCounterDisableAndReleaseCompareLogic error %d", returnValue);
            goto sample_end;
        }
        printf("\nsample_MSXE17xx__MFIncCounterDisableAndReleaseCompareLogic OK");
    }

    // Frequency measurement sample
    if (sampleNumber == 1)
    {
        uint32_t ulTimingInterval = (sampleOption02 == 0 ? 45000 : 10000);
        returnValue = dataServerFunction(&modbus, argv[1], ulMFModuleIndex, 5, ulTimingInterval, sampleOption01, sampleOption02);
        if (returnValue != 0)
            printf("\ndataServerFunction error %d", returnValue);

        returnValue = sample_MSXE17xx__MFIncCounterDisableAndReleaseFrequencyMeasurement(&modbus, ulMFModuleIndex);
        if (returnValue != 0)
        {
            printf("\nsample_MSXE17xx__MFIncCounterDisableAndReleaseFrequencyMeasurement error %d", returnValue);
            goto sample_end;
        }
    }

    // Latch sample
    if (sampleNumber == 2)
    {
        returnValue = sample_MSXE17xx__MFIncCounterInitAndEnableLatch(&modbus, ulMFModuleIndex);
        if (returnValue != 0)
        {
            printf("\nsample_MSXE17xx__MFIncCounterInitAndEnableLatch error %d", returnValue);
            goto sample_end;
        }
        returnValue = dataServerFunction(&modbus, argv[1], ulMFModuleIndex, 6 * sizeof(uint32_t), 0, sampleOption01, sampleOption02);
        if (returnValue != 0)
            printf("\ndataServerFunction error %d", returnValue);

        returnValue = sample_MSXE17xx__MFIncCounterDisableAndReleaseLatch(&modbus, ulMFModuleIndex);
        if (returnValue != 0)
        {
            printf("\nsample_MSXE17xx__MFIncCounterDisableAndReleaseLatch error %d", returnValue);
            goto sample_end;
        }    }

    // Read sample (Cyclic acquisition)
    if (sampleNumber == 3)
    {
        for(i=0;i<100;i++)
        {
            returnValue = sample_MSXE17xx__MFIncCounterRead32BitsValue(&modbus, ulMFModuleIndex, &ulValue, &ulTimeStampLow, &ulTimeStampHigh);
            if (returnValue != 0)
            {
                printf("\nsample_MSXE17xx__MFIncCounterRead32BitsValue error %d", returnValue);
                goto sample_end;
            }
#ifdef WIN32
            sleep(100);
#else
            sleep(1);
#endif
        }
    }

    // Clear sample
    if (sampleNumber == 4)
    {
        returnValue = sample_MSXE17xx__MFIncCounterInitAndEnableClear(&modbus, ulMFModuleIndex);
        if (returnValue != 0)
        {
            printf("\nsample_MSXE17xx__MFIncCounterInitAndEnableClear error %d", returnValue);
            goto sample_end;
        }
        printf("\nPress ESC to exit the sample");
        fflush(stdout);
        {
            key_thread_handle_t kt;
            unsigned int stop = 0;
            wait_key_input_thread_parameters_t ktparams;
            ktparams.character = 27; /* ESC */
            ktparams.flag = &stop;
            kt = addidata_start_key_thread(&ktparams);
            while(stop == 0)
            {
                for(i=0;i<100;i++)
                {
                    returnValue = sample_MSXE17xx__MFIncCounterRead32BitsValue(&modbus, ulMFModuleIndex, &ulValue, &ulTimeStampLow, &ulTimeStampHigh);
                    if (returnValue != 0)
                    {
                        printf("\nsample_MSXE17xx__MFIncCounterRead32BitsValue error %d", returnValue);
                        goto sample_end;
                    }
#ifdef WIN32
                    sleep(100);
#else
                    sleep(1);
#endif
                }
            }
            addidata_stop_key_thread(kt);
        }

        returnValue = sample_MSXE17xx__MFIncCounterDisableAndReleaseClear(&modbus, ulMFModuleIndex);
        if (returnValue != 0)
        {
            printf("\nsample_MSXE17xx__MFIncCounterDisableAndReleaseClear error %d", returnValue);
            goto sample_end;
        }    }

    sample_end:
    returnValue = sample_MSXE17xx__MFIncCounterReleaseHardwareTrigger(&modbus, ulMFModuleIndex);
    if (returnValue != 0)
        printf("\nsample_MSXE17xx__MFIncCounterReleaseHardwareTrigger error %d", returnValue);
    returnValue = sample_MSXE17xx__MFIncCounterReleaseIndex(&modbus, ulMFModuleIndex);
    if (returnValue != 0)
        printf("\nsample_MSXE17xx__MFIncCounterReleaseIndex error %d", returnValue);
    returnValue = sample_MSXE17xx__MFIncCounterRelease(&modbus, ulMFModuleIndex);
    if (returnValue != 0)
        printf("\nsample_MSXE17xx__MFIncCounterRelease error %d", returnValue);

    printf("\n");
    return 0;
}
