/** @file ReferenceVoltageActivation.c
 * 
 * ReferenceVoltageActivation: Permit to activate / deactivate the reference voltage on pin D- for a selected multifunction sub module
 *
 * @author C. Guinot
 * @date 19.02.2007
 * 
 * @version  $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <time.h>


#include "..\..\..\..\..\..\Interface_Library\MSXE170x.nsmap"
#include "..\..\..\UTIL\utility.h"

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

void main(void)
{
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	struct MSXE170x__Response Response;
	unsigned char pc_IPAddressTemp[50];
	unsigned char pc_IPAddress[50];
	unsigned char pc_SOAPIPAddress[50];
	DWORD dw_Dummy = 0;
	int i_Error = 0;
	int i_ReturnValue = 0;
	int i_key = 0;
	unsigned long  ulMFModuleIndex = 0;
	unsigned long  ulActivationFlag = 0;

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E170x                                         |\n"); 
 	printf("| Permit to activate / deactivate the               |\n");
    printf("| reference voltage on pin D- for a selected        |\n");
    printf("| multifunction sub module                          |\n");  
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		/* Get the IP address */
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		/* Format and set the IP address */
		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Get the module type */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, 0, "soap_call_MXCommon__GetModuleType", 1) == 0)
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);

            do
            {
                printf("Select the multifunction sub module (0 to 3):");
                scanf("%lu",&ulMFModuleIndex);
            }
            while (ulMFModuleIndex > 3);

            printf("\n");
									
            do
            {
                printf("Do You want to activate the reference voltage on pin D- (0: No, 1: Yes):");
                scanf("%lu",&ulActivationFlag);
            }
            while (ulActivationFlag > 1);
            printf("\n");

			i_Error = soap_call_MSXE170x__MFCommonReferenceVoltageActivation (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulMFModuleIndex, ulActivationFlag,0,0,&Response); 
			i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__MFCommonReferenceVoltageActivation", 0);
			
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
}
