/** @file ReadWriteClear.c
 * 
 * ReadWriteClear: Read / Write / Clear one counter
 * o	MSXE170x__CounterInit 
 * o	MSXE170x__CounterRead32BitValue 
 * o	MSXE170x__CounterWrite32BitValue 
 * o	MSXE170x__CounterClear 
 * o	MSXE170x__CounterRelease 
 *
 * @author J. Krauth
 * @date 06.11.2006
 * 
 * @version  $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <time.h>


#include "..\..\..\..\..\..\Interface_Library\MSXE170x.nsmap"
#include "..\..\..\UTIL\utility.h"

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

/** Set counter configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_CounterInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long *ulCounterIndex)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;
	unsigned long  ulCounterMode = 0;
	unsigned long  ulCounterValue = 0;
	unsigned long  ulCounterOption = 0;
	int ret = 0;

		do
		{
			printf("\nEnter the Index of the counter module (0 - 3): ");
			scanf("%lu", ulCounterIndex);
		}
		while (*ulCounterIndex > 3);

		do
		{
			printf("SIMPLE MODE		= %lu\n", MSXE170X_COUNTER_SIMPLE_MODE);
			printf("DOUBLE MODE		= %lu\n", MSXE170X_COUNTER_DOUBLE_MODE);
			printf("QUADRUPLE MODE	= %lu\n", MSXE170X_COUNTER_QUADRUPLE_MODE);
			printf("DIRECT MODE		= %lu\n", MSXE170X_COUNTER_DIRECT_MODE);
			printf("Set the counter mode for module %lu: ", *ulCounterIndex);
			scanf("%lu", &ulCounterMode);
		}
		while (	(ulCounterMode != MSXE170X_COUNTER_QUADRUPLE_MODE) && (ulCounterMode != MSXE170X_COUNTER_DOUBLE_MODE) &&
				(ulCounterMode != MSXE170X_COUNTER_SIMPLE_MODE) && (ulCounterMode != MSXE170X_COUNTER_DIRECT_MODE) );

		do
		{
			if (ulCounterMode != MSXE170X_COUNTER_DIRECT_MODE)
               {
               printf("HYSTERESIS ON = %lu\n",MSXE170X_COUNTER_HYSTERESIS_ON);
               printf("HYSTERESIS OFF = %lu\n",MSXE170X_COUNTER_HYSTERESIS_OFF);
               }
            else
               {
               printf("INCREMENT = %lu\n",MSXE170X_COUNTER_INCREMENT);
               printf("DECREMENT = %lu\n",MSXE170X_COUNTER_DECREMENT);
               }

			printf("Set the counter option for module %lu: ", *ulCounterIndex);
			scanf("%lu", &ulCounterOption);
		}
		while (	(ulCounterOption != MSXE170X_COUNTER_HYSTERESIS_ON) && (ulCounterOption != MSXE170X_COUNTER_HYSTERESIS_OFF) &&
				(ulCounterOption != MSXE170X_COUNTER_INCREMENT) && (ulCounterOption != MSXE170X_COUNTER_DECREMENT) );

	
		printf ("\n");	

		i_Error = soap_call_MSXE170x__MFIncCounterInit (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 
										 		        *ulCounterIndex, 
												        ulCounterMode, 
												        ulCounterOption,
   												        0,
                                                        0,
                                                        0,
                                                        0,
                                                        &Response); 

		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__MFIncCounterInit", 0) == 1)
		{
			ret = 1;
			goto error;
		}

		/* You can specify a value at witch the counter will began to count */ 
		do
		{
			printf("\nEnter the value to write on counter %lu: ", *ulCounterIndex);
			scanf("%lu", &ulCounterValue);
		}
		while (ulCounterValue > 4294967295UL);
		
		printf ("\n");	
	
		i_Error = soap_call_MSXE170x__MFIncCounterWrite32BitValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, *ulCounterIndex, ulCounterValue, &Response); 
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__MFIncCounterWrite32BitValue", 0) == 1)
			ret = 1;

error:		
		/* 
		 * gSOAP doesn't release automatically the memory that 
		 * it allocates to deserialize data, that's why both 
		 * following functions have to be used to release the 
		 * allocated memory.
		 */
		soap_destroy(ps_SoapEnv);
		soap_end(ps_SoapEnv);

	return ret;
}

//-------------------------------------------------------------------------------------------------------

/** Release counter.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReleaseCounter (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long ulCounterIndex)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;

		i_Error = soap_call_MSXE170x__MFIncCounterRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulCounterIndex, &Response); 
		return (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__MFIncCounterRelease", 0));
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

void main(void)
{
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	struct MSXE170x__unsignedLongTimeStampResponse TimeStampResponse;
	struct MSXE170x__Response Response;
	unsigned char pc_IPAddressTemp[50];
	unsigned char pc_IPAddress[50];
	unsigned char pc_SOAPIPAddress[50];
	DWORD dw_Dummy = 0;
	int i_Error = 0;
	int i_ReturnValue = 0;
	int i_key = 0;
	unsigned long  ulCounterIndex = 0;
	unsigned long  ulCounterValue = 0;

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E170x                                         |\n"); 
 	printf("| Read / Write / Clear one counter                  |\n");  
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		/* Get the IP address */
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		/* Format and set the IP address */
		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Get the module type */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, 0, "soap_call_MXCommon__GetModuleType", 1) == 0)
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
									
			if (i_SAMPLE_CounterInitialization (ps_SoapEnv, pc_SOAPIPAddress, &ulCounterIndex) == 0) {


				/* Start the counter */
				printf ("\nPress a key to start reading the counter.\n");
				printf ("Any key to clear the counter.\n");
				printf ("ESC to quit.\n");

				do {
					
					/* Read the counter value */
					i_Error = soap_call_MSXE170x__MFIncCounterRead32BitValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulCounterIndex, &TimeStampResponse); 
					if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, TimeStampResponse.sResponse.iReturnValue, 0, "soap_call_MSXE170x__MFIncCounterRead32BitValue", 1) == 0)
					{			
						printf ("Counter %lu = %10lu Timestamp: ", ulCounterIndex, TimeStampResponse.ulValue);
						DisplayTimeStamp (TimeStampResponse.ulTimeStampHigh,TimeStampResponse.ulTimeStampLow);
						printf ("\r");
					} 
					else
					{
						break;
					}

					/* Press any key to clear the counter */
					if (kbhit()) 
					{
						i_key = getch();
                        printf("\n");

					
						i_Error = soap_call_MSXE170x__MFIncCounterClear (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulCounterIndex, &Response); 
						if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__MFIncCounterClear", 0) == 1)
							break;
						
					}

				/* Exit on ESC */
				} while (i_key != 27);

			}
			
			i_SAMPLE_ReleaseCounter (ps_SoapEnv, pc_SOAPIPAddress, ulCounterIndex);

		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
}
