/** @file DigIOReadWriteAll.c
 * 
 * DigIOReadWriteAll: Read / Write all digital I/O
 * o	MSXE170x__DigitalIOInitPortConfiguration
 * o	MSXE170x__DigitalIOReadAllChannelsValue
 * o	MSXE170x__DigitalIOWriteAllChannelsValue
 * o	MSXE170x__DigitalIOReleasePortConfiguration
 * 
 *
 * @author C. Guinot
 * @date 13.08.2008
 * 
 * @version $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\..\Interface_Library\MSXE170x.nsmap"
#include "..\..\UTIL\utility.h"

#define SOAP_ACTION ""

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

/** Set all port configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_SetPortConfiguration (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long dw_NbrOfPort,unsigned long *pulPortConfiguration)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;
	unsigned long  ulPort = 0;

		Response.iReturnValue = 0;
        printf ("\n");	
		
		do
		{
			printf("Enter the configuration for all ports (0 = input / 1 = output): ");
			scanf("%lu", pulPortConfiguration);
			printf("\n");
		}
		while (*pulPortConfiguration > 1);

		while (((i_Error == 0) && (Response.iReturnValue == 0)) && (ulPort < dw_NbrOfPort))
		{

			i_Error = soap_call_MSXE170x__DigitalIOInitPortConfiguration(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulPort, *pulPortConfiguration, &Response);			
			if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__DigitalIOInitPortConfiguration", 1) == 0)
			{			
				printf ("Set port %lu as %s\n", ulPort, (*pulPortConfiguration==1?"output":"input"));			
			} 
			
			ulPort++;
		}	
		
		if ((i_Error == 0) && (Response.iReturnValue == 0))
			return 0;
	
	return 1;
}

//-------------------------------------------------------------------------------------------------------

/** Read all channel value.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReadAllChannelsValue (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE170x__unsignedLongResponse Response;
	int i_Error = 0;
	
        Response.sResponse.iReturnValue = 0;
        printf ("\n Channel ");	
	
		i_Error = soap_call_MSXE170x__DigitalIOReadAllChannelsValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL, &Response); 
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.sResponse.iReturnValue, 0, "soap_call_MSXE170x__DigitalIOReadAllChannelsValue", 1) == 0)
		{			
			printf ("DIG I/O Channels value 0x%lx ",Response.ulValue);			
		} 
	
		if ((i_Error == 0) && (Response.sResponse.iReturnValue == 0))
			return 0;
	
	return 1;	
}

//-------------------------------------------------------------------------------------------------------

/** Write all channels value.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_WriteAllChannelsValue (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;
	unsigned long  ulChannel = 0;
	unsigned long  ulChannelValue = 0;

        Response.iReturnValue = 0;
		printf ("\n");	
		
			
		do 
		{				
			printf ("Value to write on the channels (0 to 65535): ");
			scanf ("%lu", &ulChannelValue);
            printf("\n");
		}
		while (ulChannelValue > 65535);			
		
		i_Error = soap_call_MSXE170x__DigitalIOWriteAllChannelsValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulChannelValue, &Response); 
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__DigitalIOWriteAllChannelsValue", 1) == 0)
		{			
			printf ("Write 0x%lx on channels\n", ulChannelValue);			
		} 
		else
		{
			return 1;	
		}
	
		return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Release all port configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReleasePortConfiguration (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long dw_NbrOfPort)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;
	unsigned long  ulPort = 0;

		Response.iReturnValue = 0;
		printf ("\n");	

        while (((i_Error == 0) && (Response.iReturnValue == 0)) && (ulPort < dw_NbrOfPort))
		{
			i_Error = soap_call_MSXE170x__DigitalIOReleasePortConfiguration (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulPort, &Response); 
			if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__DigitalIOReleasePortConfiguration", 1) == 0)
			{			
				printf ("Release port %lu\n", ulPort);			
			} 
		
			ulPort++;
		}	
		
		if ((i_Error == 0) && (Response.iReturnValue == 0))
			return 0;
	
	return 1;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

void main(void)
{
	int i_Error = 0;
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	unsigned char pc_IPAddressTemp[50];
	unsigned char pc_IPAddress[50];
	unsigned char pc_SOAPIPAddress[50];
	unsigned long dw_Dummy = 0;
	unsigned long dw_NbrOfPort = 8;
    unsigned long ulPortConfiguration = 0;
    unsigned long ulExit = 0;

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E170x                                         |\n");
	printf("| DigIOReadWriteAll: Read / Write All digital I/O   |\n"); 
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		/* Get the IP address */
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		/* Format and set the IP address */
		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Get the module type */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, 0, "soap_call_MXCommon__GetModuleType", 1) == 0)
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);

			/* Set port configuration */
			if (i_SAMPLE_SetPortConfiguration (ps_SoapEnv, pc_SOAPIPAddress, dw_NbrOfPort,&ulPortConfiguration) == 0) 
			{			
				if (ulPortConfiguration == 0)
                {
                    printf("\n Press a key to exit the read of the channels ...");
                    do
                    {
						/* Read all channel value */
						if (i_SAMPLE_ReadAllChannelsValue (ps_SoapEnv, pc_SOAPIPAddress) != 0)
							break;

						/* 
						 * gSOAP doesn't release automatically the memory that 
						 * it allocates to deserialize data, that's why both 
						 * following functions have to be used to release the 
						 * allocated memory.
						 */
						soap_destroy(ps_SoapEnv);
						soap_end(ps_SoapEnv);
                    }
                    while (!kbhit());
                    getch();
                }
				else
                {
                    do
                    {
                    
                    	/* Write one channel value */
    					if (i_SAMPLE_WriteAllChannelsValue (ps_SoapEnv, pc_SOAPIPAddress) != 0)
                            break;

						/* 
						 * gSOAP doesn't release automatically the memory that 
						 * it allocates to deserialize data, that's why both 
						 * following functions have to be used to release the 
						 * allocated memory.
						 */
						soap_destroy(ps_SoapEnv);
						soap_end(ps_SoapEnv);

                        printf("Do You want to write another value on the outputs ? (0:No 1:Yes):");
                        scanf("%lu",&ulExit);
                        printf("\n");
                    }
                    while (ulExit == 1);

                }
				

			/* Release all port configuration */
			i_SAMPLE_ReleasePortConfiguration (ps_SoapEnv, pc_SOAPIPAddress, dw_NbrOfPort);
			}					
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
}
