/** @file DigIOShortCircuit.c
 * 
 * DigIOShortCircuit: digital I/O short circuit
 * o	MSXE170x__DigitalIOInitPortConfiguration
 * o	MSXE170x__DigitalIOTestShortCircuit
 * o	MSXE170x__DigitalIORearmShortCircuit
 * o	MSXE170x__DigitalIOWriteChannelValue
 * o	MSXE170x__DigitalIOReleasePortConfiguration
 * 
 *
 * @author J. Krauth
 * @date 08.01.2007
 * 
 * @version $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\..\Interface_Library\MSXE170x.nsmap"
#include "..\..\UTIL\utility.h"

#define SOAP_ACTION ""



//-------------------------------------------------------------------------------------------------------

/** Test short-circuit on digital output.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_TestShortCircuit (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE170x__unsignedLongResponse LongResponse;
	struct MSXE170x__Response Response;
	INT i_Error = 0;
	unsigned long  ulOption = 0;
    unsigned long ul_Choice = 0;
    int i_Cpt = 0;
	int ret = 0;

		LongResponse.sResponse.iReturnValue = 0;
		Response.iReturnValue = 0;
	
					
		i_Error = soap_call_MSXE170x__DigitalIOTestShortCircuit (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulOption, &LongResponse);
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, LongResponse.sResponse.iReturnValue, 0, "soap_call_MSXE170x__DigitalIOTestShortCircuit", 1) == 0)
		{			
			printf ("\r Short-circuit mask (one bit for each output) = ");
            for (i_Cpt = 15; i_Cpt >= 0;i_Cpt --)
            {
                printf("%lu",(LongResponse.ulValue >> i_Cpt) & 1 );
            }
			
			/* If a short circuit is detected */
			if (LongResponse.ulValue != 0)
			{
				do
                {
                    printf("\n A short circuit is detected, do You want to rearm the output ? (0:No, 1:Yes):");
                    scanf("%lu",&ul_Choice);
                    printf("\n");
                }
                while (ul_Choice > 1);

                if (ul_Choice == 1)
                {
                    i_Error = soap_call_MSXE170x__DigitalIORearmShortCircuit(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulOption, &Response);
					if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__DigitalIORearmShortCircuit", 0) == 1)
				    {			
					    ret = 1;	
				    }
                }
			}
		} 
		else
		{
			ret = 1;	
		}
	
		/* 
		 * gSOAP doesn't release automatically the memory that 
		 * it allocates to deserialize data, that's why both 
		 * following functions have to be used to release the 
		 * allocated memory.
		 */
		soap_destroy(ps_SoapEnv);
		soap_end(ps_SoapEnv);

		return ret;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

void main(void)
{
	INT i_Error = 0;
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	unsigned char pc_IPAddressTemp[50];
	unsigned char pc_IPAddress[50];
	unsigned char pc_SOAPIPAddress[50];
	unsigned long dw_Dummy = 0;
	unsigned long dw_NbrOfPort = 8;
	unsigned long dw_NbrOfChannel = 16;
    unsigned long dw_PortConfiguration = 0;

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E170x                                         |\n");
 	printf("| DigIOShortCircuit: digital I/O short circuit      |\n"); 
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		/* Get the IP address */
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		/* Format and set the IP address */
		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Get the module type */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, 0, "soap_call_MXCommon__GetModuleType", 1) == 0)
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);

    		printf ("Press ESC to quit \n");	
			/* Test if an output is in short-circuit */
			do
            {
                i_SAMPLE_TestShortCircuit(ps_SoapEnv, pc_SOAPIPAddress);
            }
            while (!kbhit());
            
            
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");
    getch();

	while (!(kbhit()));
}
