/** @file DigIOWatchdog.c
 * 
 * DigIOWatchdog: Digital I/O watchdog
 * o	MSXE170x__DigitalIOInitPortConfiguration
 * o	MSXE170x__IOWatchdogInitAndStart 
 * o	MSXE170x__DigitalIOWriteChannelValue
 * o	MSXE170x__IOWatchdogGetStatusAndValue 
 * o	MSXE170x__IOWatchdogStopAndRelease
 * o	MSXE170x__DigitalIOReleasePortConfiguration
 * 
 *
 * @author J. Krauth
 * @date 08.01.2007
 * 
 * @version $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "..\..\..\..\..\Interface_Library\MSXE170x.nsmap"
#include "..\..\UTIL\utility.h"


#define SOAP_ACTION ""

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

/** Set all port configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_SetPortConfiguration (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long dw_NbrOfPort)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;
	unsigned long  ulPort = 0;
	unsigned long  ulPortConfiguration = 1; /* Set the port as output */

		Response.iReturnValue = 0;
	
		i_Error = soap_call_MSXE170x__DigitalIOInitPortConfiguration(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulPort, ulPortConfiguration, &Response);			
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__DigitalIOInitPortConfiguration", 1) == 0)
		{			
			printf ("Set port %lu as %s\n", ulPort, (ulPortConfiguration==1?"output":"input"));			
			return 0;
		} 
	
	
	return 1;
}

//-------------------------------------------------------------------------------------------------------

/** Initialize and startthe watchdog.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_WatchdogInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;
	unsigned long  ulChannel = 0;
	unsigned long  ulTimeBase = 0;
	unsigned long  ulTimeValue = 0;
	
		Response.iReturnValue = 0;
        printf ("Watchdog configuration\n");	

		do 
		{				
			printf ("Time base of the watchdog delay (0 for us, 1 for ms, 2 for s): ");
			scanf ("%lu", &ulTimeBase);
            printf("\n");
		}
		while (ulTimeBase > 2);	

		do 
		{				
			printf ("Time base value of the watchdog delay (0 to 0xFFFF): ");
			scanf ("%lu", &ulTimeValue);
            printf("\n");
		}
		while (ulTimeValue > 0xFFFF);	

        /* Stop the watchdog in case it is already started */
		i_Error = soap_call_MSXE170x__IOWatchdogStopAndRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 0, &Response);
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE170x__IOWatchdogStopAndRelease OK\n");			
		} 


	
		i_Error = soap_call_MSXE170x__IOWatchdogInitAndStart (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulTimeBase, ulTimeValue, 0, 0, &Response);
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__IOWatchdogInitAndStart", 0) == 1)
		{
			return 1;	
		}
			
	
	return 0;	
}

//-------------------------------------------------------------------------------------------------------

/** Write one channel value.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_WriteChannelValue (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long dw_NbrOfChannel)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;
	unsigned long  ulChannel = 0;
	unsigned long  ulChannelValue = 1;

		Response.iReturnValue = 0;
	
		printf ("\n");	
				
		i_Error = soap_call_MSXE170x__DigitalIOWriteChannelValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulChannel, ulChannelValue, &Response); 
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__DigitalIOWriteChannelValue", 1) == 0)
		{			
			printf ("Write %lu on channel %lu\n", ulChannelValue, ulChannel);			
		} 
		else
		{
			return 1;	
		}
	
		return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Get watchdog status and value.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_GetWatchdogStatusAndValue (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE170x__IOWatchdogGetStatusAndValueResponse WatchdogStatusAndValueResponse;
	int i_Error = 0;
	unsigned long  ulChannel = 0;
	unsigned long  ulChannelValue = 1;
    char * pc_WatchdogStatus []={"STOPPED","RUNNING"};
    char * pc_WatchdogOverStatus []={"NOT RUN DOWN","RUN DOWN"};
/*
BIN XXXXXXXX XXXXXXXX XXXXXXXX XXXXXXX0: is stopped, 
BIN XXXXXXXX XXXXXXXX XXXXXXXX XXXXXXX1: is running, 
BIN XXXXXXXX XXXXXXXX XXXXXXXX XXXXXX0X: is not run down 
BIN XXXXXXXX XXXXXXXX XXXXXXXX XXXXXX1X: is run down 
*/

		WatchdogStatusAndValueResponse.sResponse.iReturnValue = 0;
	
		printf ("\n");
					
		i_Error = soap_call_MSXE170x__IOWatchdogGetStatusAndValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 0, &WatchdogStatusAndValueResponse); 
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, WatchdogStatusAndValueResponse.sResponse.iReturnValue, 0, "soap_call_MSXE170x__IOWatchdogGetStatusAndValue", 1) == 0)
		{			
			printf ("Watchdog status %lu : %s,%s value = %lu ", WatchdogStatusAndValueResponse.ulStatus,
                                                                pc_WatchdogStatus[WatchdogStatusAndValueResponse.ulStatus & 1],
                                                                pc_WatchdogOverStatus[(WatchdogStatusAndValueResponse.ulStatus & 2) >> 1],
                                                                WatchdogStatusAndValueResponse.ulValue);			
		} 
		else
		{
			return 1;	
		}
	
		return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Release all port configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReleasePortConfiguration (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long dw_NbrOfPort)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;
	unsigned long  ulPort = 0;

		printf ("\n");	
	
		Response.iReturnValue = 0;
		i_Error = soap_call_MSXE170x__DigitalIOReleasePortConfiguration (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulPort, &Response); 
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__DigitalIOReleasePortConfiguration", 1) == 0)
		{			
			printf ("Release port %lu\n", ulPort);			
			return 0;
		} 
		
	
	return 1;
}

//-------------------------------------------------------------------------------------------------------

/** Stop and release the watchdog.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_StopAndReleaseWatchdog (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE170x__Response Response;
	int i_Error = 0;

		printf ("\n");	
	
		Response.iReturnValue = 0;

		i_Error = soap_call_MSXE170x__IOWatchdogStopAndRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 0, &Response); 
		return (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, Response.iReturnValue, 0, "soap_call_MSXE170x__IOWatchdogStopAndRelease", 0));
		
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

void main(void)
{
	int i_Error = 0;
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	unsigned char pc_IPAddressTemp[50];
	unsigned char pc_IPAddress[50];
	unsigned char pc_SOAPIPAddress[50];
	unsigned long dw_Dummy = 0;
	unsigned long dw_NbrOfPort = 8;
	unsigned long dw_NbrOfChannel = 16;

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E170x                                         |\n");
 	printf("| DigIOWatchdog: Digital I/O watchdog               |\n"); 
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		/* Get the IP address */
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		/* Format and set the IP address */
		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Get the module type */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if (i_CheckAndPrintError (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, i_Error, s_ByteArrayResponse.sResponse.iReturnValue, 0, "soap_call_MXCommon__GetModuleType", 1) == 0)
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);

			/* Set port configuration */
			if (i_SAMPLE_SetPortConfiguration (ps_SoapEnv, pc_SOAPIPAddress, dw_NbrOfPort) == 0) 
			{
				/* Initialize and start the watchdog */
				if (i_SAMPLE_WatchdogInitialization (ps_SoapEnv, pc_SOAPIPAddress) == 0) 
				{	

					printf ("\n Press a key to start");
					printf ("\n Once started, press any key to set the first output to 1 (trigger the watchdog) or ESC to quit\n");
					getch();
                    printf("\n");
					
					for ( ; ; )
					{

						if (i_SAMPLE_GetWatchdogStatusAndValue (ps_SoapEnv, pc_SOAPIPAddress) == 0)
						{
							/* Press any key to set all output to 1 */
							if (kbhit()) 
							{
								if (getch() == 27)
									break;

								i_SAMPLE_WriteChannelValue (ps_SoapEnv, pc_SOAPIPAddress, dw_NbrOfChannel);					
							}					
						}

						/* 
						 * gSOAP doesn't release automatically the memory that 
						 * it allocates to deserialize data, that's why both 
						 * following functions have to be used to release the 
						 * allocated memory.
						 */
						soap_destroy(ps_SoapEnv);
						soap_end(ps_SoapEnv);
					}

					/* Stop and release the watchdog */
					i_SAMPLE_StopAndReleaseWatchdog (ps_SoapEnv, pc_SOAPIPAddress);
				}

			/* Release the port configuration */
			i_SAMPLE_ReleasePortConfiguration (ps_SoapEnv, pc_SOAPIPAddress, dw_NbrOfPort);
			}					
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
}
